import 'package:get/get.dart';
import 'package:nsg_data/nsg_data.dart';

class NsgDataController<T extends NsgDataItem> extends NsgBaseController {
  List<T> get items => dataItemList.cast<T>();
  T get firstItem => (dataItemList.isEmpty) ? NsgDataClient.client.getNewObject(dataType) as T : items[0];
  T get currentItem => ((selectedItem ?? NsgDataClient.client.getNewObject(dataType)) as T);
  set currentItem(T item) => selectedItem = item;

  NsgDataController(
      {bool requestOnInit = true,
      bool useUpdate = false,
      bool useChange = true,
      List<String>? builderIDs,
      NsgBaseController? masterController,
      NsgDataBinding? dataBindign,
      bool autoRepeate = false,
      int autoRepeateCount = 10,
      bool useDataCache = false,
      bool selectedMasterRequired = true,
      bool autoSelectFirstItem = false,
      List<NsgBaseController>? dependsOnControllers})
      : super(
            dataType: T,
            requestOnInit: requestOnInit,
            useUpdate: useUpdate,
            useChange: useChange,
            builderIDs: builderIDs,
            masterController: masterController,
            dataBinding: dataBindign,
            autoRepeate: autoRepeate,
            autoRepeateCount: autoRepeateCount,
            selectedMasterRequired: selectedMasterRequired,
            useDataCache: useDataCache,
            autoSelectFirstItem: autoSelectFirstItem,
            dependsOnControllers: dependsOnControllers);

  ///Сделать текущим предыдущий элемент
  void gotoPrevItem() {
    var index = items.indexOf(currentItem);
    if (index == 0) return;
    currentItem = items[index - 1];
    refreshSelectedItem(null);
  }

  ///Сделать текущим следующий элемент
  void gotoNextItem() {
    var index = items.indexOf(currentItem);
    if (index >= items.length - 1) return;
    currentItem = items[index + 1];
    refreshSelectedItem(null);
  }

  ///Есть ли в списке элементы до текущего
  bool canGoPrevItem() {
    var index = items.indexOf(currentItem);
    return index > 0;
  }

  ///Есть ли в списке элементы после текущего
  bool canGoNextItem() {
    var index = items.indexOf(currentItem);
    return index >= 0 && index < items.length - 1;
  }

  ///Создает новый элемент. Используется, например, при нажатии добавить в форме списка
  ///На время создания (так как оно может быть связано с запросом на сервер) устанавливает статус контроллера в loading
  ///Для непосредственного создания нового элемента вызывает асинхронный метод doCreateNewItem, который может быть перекрыт
  ///для организации бизнес-логики запросов
  Future<T> createNewItem() async {
    change(null, status: RxStatus.loading());
    try {
      var elem = await doCreateNewItem();
      currentItem = elem;
      change(null, status: RxStatus.success());
      return elem;
    } catch (e) {
      var msg = '';
      if (e is NsgApiException && e.error.message != null) {
        msg = e.error.message!;
      }
      change(null, status: RxStatus.error(msg));
    }
    return NsgDataClient.client.getNewObject(dataType) as T;
  }

  ///Создает новый элемент. Вызывается из createNewItem
  ///Может быть перекрыт для организации бизнес-логики запросов, например, заполнения нового элемента на сервере
  ///или проверки возможности создания нового элемента
  Future<T> doCreateNewItem() async {
    var elem = NsgDataClient.client.getNewObject(dataType) as T;
    //Если выставлен признак создавать на сервере, создаем запрос на сервер
    if (elem.createOnServer) {
      var request = NsgDataRequest<T>();
      return await request.requestItem(method: 'POST', function: elem.apiRequestItems + '/Create');
    }
    return elem;
  }
}
